# -*- coding: utf-8 -*-
from __future__ import absolute_import
from __future__ import print_function

import xml.etree.ElementTree as ET
import cairo


units = ['em', 'ex', 'px', 'pt', 'pc', 'cm', 'mm', 'in', 'percentages']


def get_extents(text, font):
    surface = cairo.ImageSurface(cairo.FORMAT_A1, 200, 200)
    cr = cairo.Context(surface)
    cr.select_font_face(
        font,
        cairo.FONT_SLANT_NORMAL,
        cairo.FONT_WEIGHT_NORMAL,
        )
    return cr.text_extents(text)


def value_unit(value_and_unit):
    for unit in units:
        if value_and_unit.endswith(unit):
            value = float(value_and_unit[:-len(unit)])
            return value, unit


class TextObject():

    def __init__(self, x, y, r, font, size, text):
        self.x = x
        self.y = y
        self.r = r
        self.font = font
        self.size = size
        self.text = text

    def __str__(self):
        return 'T %.3f,%.3f,%s,%s,%.3f;%s' % (
            self.x, self.y, self.r, self.font, self.size, self.text
            )


def parse_style(style):
    retval = {}
    for section in style.split(';'):
        try:
            key, value = section.split(':', 1)
            retval[key] = value
        except Exception:
            print('section:', section)
    return retval


def transform_x_y(transform, x_value, y_value):
    x_value = float(x_value)
    y_value = float(y_value)
    if transform.startswith('translate(') and transform.endswith(')'):
        translate = transform[10:-1]
        if ',' in translate:
            translate = translate.split(',')
            return (
                x_value + float(translate[0]),
                y_value + float(translate[1]),
                )
        else:
            return (
                x_value + float(translate),
                y_value,
                )
    elif transform.startswith('matrix(') and transform.endswith(')'):
        a, b, c, d, e, f = (float(x) for x in transform[7: -1].split(','))
        return (
            a * x_value + c * y_value + e,
            b * x_value + d * y_value + f,
            )
    elif transform.startswith('scale(') and transform.endswith(')'):
        scale = transform[6:-1]
        if ',' in scale:
            scale = scale.split(',')
            return (
                x_value * float(scale[0]),
                y_value * float(scale[1]),
                )
        else:
            return (
                x_value * float(scale),
                y_value * float(scale),
                )
    else:
        raise ValueError(transform)


class Transform(list):
    def __init__(self, transform_list):
        super(Transform, self).__init__()
        self.extend(transform_list.split(' '))

    def x_y(self, x_value, y_value):
        for transform in self:
            # print 'transform:', transform
            x_value, y_value = transform_x_y(transform, x_value, y_value)
        return x_value, y_value

    def rotation(self):
        if 'matrix(0,-1,1,0,0,0)' in self:
            return 90
        elif 'matrix(0,1,-1,0,0,0)' in self:
            return -90
        elif 'scale(-1,-1)' in self:
            return 180
        else:
            return 0


class SvgParser():
    """
    if bbb.text == 'g':
        print 'aaa tag:', aaa.tag
        print 'aaa attrib:', aaa.attrib
        print 'bbb text:', bbb.text
        for ccc in aaa:
            #print 'text:', aaa.text
            print 'ccc tag:', ccc.tag
            print 'ccc attrib:', ccc.attrib
            for ddd in ccc:
                print 'ddd tag:', ddd.tag
                print 'ddd attrib:', ddd.attrib
                if ddd.tag == '{http://www.w3.org/2000/svg}rect':
                    print {
                        'x:': ddd.attrib['x'],
                        'height:': ddd.attrib['height'],
                        'width:': ddd.attrib['width'],
                        'y:': ddd.attrib['y'],
                        'text:': bbb.text,
                        'transform:': aaa.attrib['transform']
                        }

    """

    def __init__(self, file_name):
        print('file_name:', file_name)
        tree = ET.parse(file_name)
        svg = tree.getroot()
        self.height, height_unit = value_unit(svg.attrib['height'])
        self.width, width_unit = value_unit(svg.attrib['width'])
        if height_unit == width_unit:
            self.unit = height_unit
        else:
            raise ValueError('height_unit does not equal width_unit')
        self.viewBox = [float(i) for i in svg.attrib['viewBox'].split(' ')]
        graphic = svg.findall(".//{http://www.w3.org/2000/svg}g")[0]
        graphic_transform = (
            Transform(graphic.attrib['transform'])
            if 'transform' in graphic.attrib else None
            )
        for aaa in graphic:
            if aaa.tag == '{http://www.w3.org/2000/svg}text':
                # print 'aaa tag:', aaa.tag
                #
                # "font-style:normal;font-variant:normal;font-weight:normal;
                # font-stretch:normal;font-size:88.13702393px;line-height:125%;
                # font-family:'Swiss 721';
                # -inkscape-font-specification:'Swiss 721,
                # Normal';text-align:start;letter-spacing:0px;
                # word-spacing:0px;writing-mode:lr-tb;text-anchor:start;
                # fill:#000000;fill-opacity:1;stroke:none;
                # stroke-width:1px;stroke-linecap:butt;
                # stroke-linejoin:miter;stroke-opacity:1"
                style = parse_style(aaa.attrib['style'])
                # print 'style:', style
                for bbb in aaa:
                    if bbb.tag == '{http://www.w3.org/2000/svg}tspan':
                        # print 'bbb tag:', bbb.tag
                        # print 'bbb attrib:', bbb.attrib
                        # print 'bbb text:', bbb.text
                        text_transform = (
                            Transform(aaa.attrib['transform'])
                            if 'transform' in aaa.attrib else None
                            )
                        x, y = bbb.attrib['x'], bbb.attrib['y']
                        if text_transform:
                            x, y = text_transform.x_y(x, y)
                        if graphic_transform:
                            x, y = graphic_transform.x_y(x, y)
                        text_object = TextObject(
                            self.get_x_mm(x),
                            self.get_y_mm(y),
                            text_transform.rotation() if text_transform else 0,
                            '3',
                            self.get_y_mm(
                                float(style['font-size'][:-2])
                                ),
                            bbb.text
                            )
                        print(text_object)

    def get_mm(self, value):
        if self.unit == 'mm':
            return value
        raise ValueError('unit %s is not yet supported' % self.unit)

    def get_x_mm(self, x_value):
        return self.get_mm(float(x_value) * (self.width / self.viewBox[2]))

    def get_y_mm(self, y_value):
        return self.get_mm(float(y_value) * (self.height / self.viewBox[3]))


if __name__ == '__main__':
    for svg_file in [
        '../svg/test1.svg',
        '../svg/test90.svg',
        '../svg/test270.svg',
        '../svg/test180.svg',
        '../svg/DOC-1000101.svg',
            ]:
        SvgParser(svg_file)
